package net.sourceforge.nattable.edit.config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.sourceforge.nattable.data.convert.ConversionFailedException;
import net.sourceforge.nattable.data.validate.ValidationFailedException;
import net.sourceforge.nattable.edit.editor.AbstractCellEditor;
import net.sourceforge.nattable.edit.editor.AbstractEditErrorHandler;
import net.sourceforge.nattable.edit.editor.ICellEditor;
import net.sourceforge.nattable.edit.editor.IEditErrorHandler;

/**
 * Error handling strategy that simply logs if 
 * 
 * @author fipro
 *
 */
public class LoggingErrorHandling extends AbstractEditErrorHandler {

	private static final Log log = LogFactory.getLog(AbstractCellEditor.class);

	public LoggingErrorHandling() {
		super(null);
	}
	
	public LoggingErrorHandling(IEditErrorHandler underlyingErrorHandler) {
		super(underlyingErrorHandler);
	}
	
	@Override
	public void displayError(ICellEditor cellEditor, Exception e) {
		super.displayError(cellEditor, e);
		//for ConversionFailedException and ValidationFailedException we only want to log the corresponding
		//message. Otherwise we need the whole stack trace to find unexpected exceptions
		if (!(e instanceof ConversionFailedException) && !(e instanceof ValidationFailedException)) {
			log.warn("Error whilst converting/validating cell value", e);
		}
		else {
			log.warn("Error whilst converting/validating cell value: " + e.getLocalizedMessage());
		}
	}
}
