package net.sourceforge.nattable.edit.config;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Text;

import net.sourceforge.nattable.edit.editor.AbstractEditErrorHandler;
import net.sourceforge.nattable.edit.editor.ControlDecorationProvider;
import net.sourceforge.nattable.edit.editor.ICellEditor;
import net.sourceforge.nattable.edit.editor.IEditErrorHandler;
import net.sourceforge.nattable.edit.editor.TextCellEditor;
import net.sourceforge.nattable.util.GUIHelper;

public class RenderErrorHandling extends AbstractEditErrorHandler {

	private Color originalColor;
	protected final ControlDecorationProvider decorationProvider;
	
	public RenderErrorHandling() {
		this(null, new ControlDecorationProvider());
	}
	
	public RenderErrorHandling(ControlDecorationProvider decorationProvider) {
        this(null, decorationProvider);
	}
	
    public RenderErrorHandling(IEditErrorHandler underlyingErrorHandler, ControlDecorationProvider decorationProvider) {
		super(underlyingErrorHandler);
        this.decorationProvider = decorationProvider;
	}
	
	@Override
	public void removeError(ICellEditor cellEditor) {
		super.removeError(cellEditor);
		if (cellEditor instanceof TextCellEditor) {
			TextCellEditor textCellEditor = (TextCellEditor) cellEditor;
			Text textControl = textCellEditor.getTextControl();

			if (originalColor != null) {
				textControl.setForeground(originalColor);
				originalColor = null;
			}
			decorationProvider.hideDecoration();
		}
	}
	
	@Override
	public void displayError(ICellEditor cellEditor, Exception e) {
		super.displayError(cellEditor, e);
		if (cellEditor instanceof TextCellEditor) {
			TextCellEditor textCellEditor = (TextCellEditor) cellEditor;
			Text textControl = textCellEditor.getTextControl();
			
			if (originalColor == null) {
				originalColor = textControl.getForeground();
			}
			textControl.setForeground(GUIHelper.COLOR_RED);
			decorationProvider.showDecoration();
		}
	}

}
