package net.sourceforge.nattable.extension.glazedlists.groupBy;

import net.sourceforge.nattable.config.AbstractRegistryConfiguration;
import net.sourceforge.nattable.config.CellConfigAttributes;
import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.data.IDataProvider;
import net.sourceforge.nattable.extension.glazedlists.groupBy.action.GroupByDragMode;
import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.reorder.action.ColumnReorderDragMode;
import net.sourceforge.nattable.style.DisplayMode;
import net.sourceforge.nattable.ui.action.AggregateDragMode;
import net.sourceforge.nattable.ui.action.CellDragMode;
import net.sourceforge.nattable.ui.binding.UiBindingRegistry;
import net.sourceforge.nattable.ui.matcher.MouseEventMatcher;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;

public class GroupByHeaderConfiguration extends AbstractRegistryConfiguration {
	
	private final GroupByHeaderPainter groupByHeaderPainter;

	public GroupByHeaderConfiguration(GroupByModel groupByModel, IDataProvider columnHeaderDataProvider) {
		groupByHeaderPainter = new GroupByHeaderPainter(groupByModel, columnHeaderDataProvider);
	}
	
	public void configureRegistry(IConfigRegistry configRegistry) {
		configRegistry.registerConfigAttribute(CellConfigAttributes.CELL_PAINTER, groupByHeaderPainter, DisplayMode.NORMAL, GroupByHeaderLayer.GROUP_BY_REGION);
	}
	
	@Override
	public void configureUiBindings(UiBindingRegistry uiBindingRegistry) {
		uiBindingRegistry.registerFirstMouseDragMode(
				MouseEventMatcher.columnHeaderLeftClick(SWT.NONE),
				new AggregateDragMode(
						new CellDragMode(),
						new ColumnReorderDragMode() {
							@Override
							protected boolean isValidTargetColumnPosition(
									ILayer natLayer,
									int dragFromGridColumnPosition,
									int dragToGridColumnPosition,
									MouseEvent event) {
								// Suppress reorder if cursor is over the group by region
								if (!natLayer.getRegionLabelsByXY(event.x, event.y).hasLabel(GroupByHeaderLayer.GROUP_BY_REGION)) {
									return super.isValidTargetColumnPosition(natLayer, dragFromGridColumnPosition,
											dragToGridColumnPosition, event);
								}
								return false;
							}
						},
						new GroupByDragMode()));
	}
	
	public GroupByHeaderPainter getGroupByHeaderPainter() {
		return groupByHeaderPainter;
	}
	
}
