package net.sourceforge.nattable.ui.action;

import net.sourceforge.nattable.NatTable;
import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.painter.IOverlayPainter;
import net.sourceforge.nattable.painter.cell.ICellPainter;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Rectangle;

public class CellDragMode implements IDragMode {
	
	private MouseEvent initialEvent;
	private MouseEvent currentEvent;
	
	private int xOffset;
	private int yOffset;
	private Image cellImage;
	protected CellImageOverlayPainter cellImageOverlayPainter = new CellImageOverlayPainter();
	
	public void mouseDown(NatTable natTable, MouseEvent event) {
		initialEvent = event;
		currentEvent = initialEvent;
		
		setCellImage(natTable);
		
		natTable.forceFocus();

		natTable.addOverlayPainter(cellImageOverlayPainter);
	}

	public void mouseMove(NatTable natTable, MouseEvent event) {
		currentEvent = event;
		
		natTable.redraw(0, 0, natTable.getWidth(), natTable.getHeight(), false);
	}

	public void mouseUp(NatTable natTable, MouseEvent event) {
		natTable.removeOverlayPainter(cellImageOverlayPainter);
		cellImage.dispose();
		
		natTable.redraw(0, 0, natTable.getWidth(), natTable.getHeight(), false);
	}
	
	protected MouseEvent getInitialEvent() {
		return initialEvent;
	}
	
	protected MouseEvent getCurrentEvent() {
		return currentEvent;
	}
	
	private void setCellImage(NatTable natTable) {
		int columnPosition = natTable.getColumnPositionByX(currentEvent.x);
		int rowPosition = natTable.getRowPositionByY(currentEvent.y);
		LayerCell cell = natTable.getCellByPosition(columnPosition, rowPosition);
		
		Rectangle cellBounds = cell.getBounds();
		xOffset = currentEvent.x - cellBounds.x;
		yOffset = currentEvent.y - cellBounds.y;
		Image image = new Image(natTable.getDisplay(), cellBounds.width, cellBounds.height);
		
		GC gc = new GC(image);
		IConfigRegistry configRegistry = natTable.getConfigRegistry();
		ICellPainter cellPainter = cell.getLayer().getCellPainter(columnPosition, rowPosition, cell, configRegistry);
		if (cellPainter != null) {
			cellPainter.paintCell(cell, gc, new Rectangle(0, 0, cellBounds.width, cellBounds.height), configRegistry);
		}
		gc.dispose();

		ImageData imageData = image.getImageData();
		image.dispose();
		imageData.alpha = 150;
		
		cellImage = new Image(natTable.getDisplay(), imageData);
	}

	private class CellImageOverlayPainter implements IOverlayPainter {

		public void paintOverlay(GC gc, ILayer layer) {
			if (cellImage != null & !cellImage.isDisposed()) {
				gc.drawImage(cellImage, currentEvent.x - xOffset, currentEvent.y - yOffset);
			}
		}

	}

}
