package net.sourceforge.nattable.ui.matcher;

import net.sourceforge.nattable.NatTable;
import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.layer.LabelStack;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.painter.cell.ICellPainter;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

/**
 * Matches a mouse click on a given cell painter within a cell.
 */
public class CellPainterMouseEventMatcher extends CellLabelMouseEventMatcher {

	private final ICellPainter targetCellPainter;

	public CellPainterMouseEventMatcher(String regionName, int button, String labelToMatch, ICellPainter targetCellPainter) {
		super(regionName, button, labelToMatch);
		this.targetCellPainter = targetCellPainter;
	}

	@Override
    public boolean matches(NatTable natTable, MouseEvent event, LabelStack regionLabels) {
		if (super.matches(natTable, event, regionLabels)) {
			int columnPosition = natTable.getColumnPositionByX(event.x);
			int rowPosition = natTable.getRowPositionByY(event.y);
			
			LayerCell cell = natTable.getCellByPosition(columnPosition, rowPosition);
			IConfigRegistry configRegistry = natTable.getConfigRegistry();
			ICellPainter cellPainter = cell.getLayer().getCellPainter(columnPosition, rowPosition, cell, configRegistry);
			
			GC gc = new GC(natTable.getDisplay());
			try {
				Rectangle adjustedCellBounds = natTable.getLayerPainter().adjustCellBounds(cell.getBounds());
				
				ICellPainter clickedCellPainter = cellPainter.getCellPainterAt(event.x, event.y, cell, gc, adjustedCellBounds, configRegistry);
				if (clickedCellPainter == targetCellPainter) {
					return true;
				}
			} finally {
				gc.dispose();
			}
		}
		return false;
	}

}
